# -*- coding: UTF-8 -*-
"""
Module chargé de gérer les fenêtres liées aux directives.
"""

import gtk, gtk.glade
# import gobject

from era.noyau.dpatterns import Observer, Singleton
from era.noyau import fwobjects
from gtk_models import create_objects_treestore, create_extremite_treemodel, create_directive_treemodel

from ihm_utils import *
from era.noyau.pool import library_store
from era.noyau.erreurs import BadPortError, TagError

class DirectiveSummaryView(Observer):
    """ Vue pour l'ensemble des règles
    """

    def __init__(self, glade_file, ctrl, flux, directive_type):
        """
        glade : l'objet XML glade
        ctrl : le controller pour cette vue
        """
        self.glade_file = glade_file
        self.glade = gtk.glade.XML(self.glade_file,"directive_summary_dialog", "editeur")
        self.ctrl = ctrl
        self.directive_type = directive_type
        # Création de la vue et du modèle
        if flux is None:
            self.set_model(fwobjects.DirectiveStore())
        elif directive_type == fwobjects.DIRECTIVE_TYPE_UP:
            self.set_model(flux.up_directives_model())
        else:
            self.set_model(flux.down_directives_model())
        self.flux = flux
        # XXX FIXME : BEURK !
        self.ctrl.model = self.model
        self.gtk_model = create_directive_treemodel()
        # la fenêtre principale
        self.main_window = self.glade.get_widget('directive_summary_dialog')
        # affichage des zones du directive store
        self.label = self.glade.get_widget('directive_summary_label')
        try:
            if self.directive_type==1:
                # self.label.set_text("source : %s, destination : %s" % (flux.zoneA.name,flux.zoneB.name))
                self.main_window.set_title(_('directive summary') + ' : %s -> %s' % (flux.zoneA.name,flux.zoneB.name))
            else:
                # self.label.set_text("source : %s, destination : %s" % (flux.zoneB.name,flux.zoneA.name))
                self.main_window.set_title(_('directive summary') + ' : %s -> %s' % (flux.zoneB.name,flux.zoneA.name))
        except Exception,e:
            self.label.set_text("")
        # la liste des règles
        self.treeview = self.glade.get_widget('directive_summary_treeview')

        # initialisation des données du treeview
        self.init_treeview()
        self.ctrl.gtk_model = self.gtk_model
        self.fill_treeview()

        self.treeview.set_headers_visible(1)
        handlers = {
            "on_directive_summary_dialog_response" : self.hide_window,
            "on_directive_new_item_activate" : (self.ctrl.add_directive, self.flux, self.directive_type),
            "on_directive_del_item_activate" : (self.ctrl.del_directive, self.treeview, self, self.main_window),
            "on_close_item_activate" : self.hide_window,
            "on_directive_summary_dialog_close" : self.hide_window,
            "on_directive_summary_delete_event" : self.hide_window,
            "on_directive_summary_down_button_clicked" : (self.ctrl.directive_down, self.treeview),
            "on_directive_summary_up_button_clicked" : (self.ctrl.directive_up, self.treeview),
            "on_directive_summary_treeview_button_press_event" : self.ctrl.button_pressed,
            "on_directive_summary_add_button_clicked" : (self.ctrl.add_directive, self.flux, self.directive_type),
            }

        self.glade.signal_autoconnect(handlers)

    def init_treeview(self):
        # création du treeview des directives
        col_names = [_("PRIORITY"), _("SRC"), _("DEST"), _("SERVICE"), _("ACTION"), _("TIMERANGE"), _("USER_GROUP"),
                     _("MARK"), _("LOG"), _("OPTIONAL")]

        col_index = 0
        for col_name in col_names:
            if col_index < 8:
                renderer = gtk.CellRendererText()
                column = gtk.TreeViewColumn(col_name, renderer, text = col_index)
            else:
                renderer = gtk.CellRendererToggle()
                column = gtk.TreeViewColumn(col_name, renderer, active = col_index)
            self.treeview.append_column(column)
            col_index += 1

        self.treeview.set_model(self.gtk_model)

    def fill_treeview(self):
        for directive in self.model.directive_list():
            self.gtk_model.append(directive.as_list())

    def set_model(self, model):
        """
        """
        self.model = model
        self.model.register_observer(self)


    def show_window(self):
        """ Affiche la fenêtre de dialogue
        """
        self.main_window.show_all()


    def hide_window(self, *args):
        """ Cache la fenêtre de dialogue
        """
        self.main_window.hide()
        return False


    def update(self):
        """ Mets à jour la liste des directives affichées
        """
        pass

class DirectiveManager:
    """ Classe qui joue le rôle du Controller pour la gestion des directives.
    """

    def __init__(self, glade_file, model = None):
        """
        glade : l'objet XML glade
        """
        self.glade_file = glade_file
        # self.model = fwobjects.DirectiveStore()
        self.model = model
        self.gtk_model = None


    def add_directive(self, item,  flux, directive_type):
        """ Lance la fenêtre d'édition de règle
        """
        directive_dlg = DirectiveDialog(self.glade_file, self.model,
                                        flux.zoneA, flux.zoneB,
                                        directive_type, self)
        directive_dlg.show_dialog()

    def del_directive(self,item,treeview,view,dialog,*args):
        """Supprime la directive selectionnée
        """
        selection = treeview.get_selection()
        store, iter_sel = selection.get_selected()
        if iter_sel is not None:
            # on vérifie si la directive est éditable ou non (directives importées)
            index = store.get_path(iter_sel)[0]
            selected_dir = store[index][-1]
            if selected_dir.is_editable() == 0:
                dial_alerte=create_error_dialog(_('non editable selection'))
                dial_alerte.show()
            else:
                # demande de confirmation et suppression de la directive
                dial_conf = create_yes_no_dialog(_('remove confirmation'))
                # dial_conf=gtk.MessageDialog(dialog,gtk.DIALOG_MODAL,buttons=gtk.BUTTONS_YES_NO)
                # dial_conf.label.set_text(
                dial_conf.set_default_response(gtk.RESPONSE_NO)
                ret = dial_conf.run()
                dial_conf.destroy()
                # on teste le retour du dialogue
                if ret == gtk.RESPONSE_YES:
                    # suppression dans le classeur de directives
                    # libération des services et extremites
                    selected_dir.service.used -= 1
                    for extr in selected_dir.src_list:
                        extr.used -= 1
                    for extr in selected_dir.dest_list:
                        extr.used -= 1
                    if selected_dir.time != None:
                        selected_dir.time.used -= 1
                    if selected_dir.user_group != None:
                        selected_dir.user_group.used -= 1
                    self.model.del_directive(index)
                    # mise à jour de l'affichage
                    store.remove(iter_sel)
                    self.gtk_model.clear()
                    view.fill_treeview()


    def swap(self, index1, index2):
        """Echange l'ordre des deux directives pointées par index1 et index2
        """
        row1 = self.gtk_model[index1]
        row2 = self.gtk_model[index2]
        col_index = 1
        # On inverse l'ensemble des contenus des colonnes, sauf pour la première
        # qui représente la priorité
        # XXX on ne peut pas faire row[1:] sur un TreeRowModel
        for col_index in range(1,len(row1)):
            val = row1[col_index]
            row1[col_index] = row2[col_index]
            row2[col_index] = val

        self.model.swap(index1, index2)


    # XXX : ListStore.swap() n'est pas défini
    def directive_up(self, button, treeview):
        """Callback appelée pour remonter une directive
        """
        selection = treeview.get_selection()
        model, iter_up = selection.get_selected()
        line_index = model.get_path(iter_up)[0]
        if line_index > 0:
            self.swap(line_index-1, line_index)
            # on sélectionne la ligne à sa nouvelle position
            path_prev=model.get_path(iter_up)
            path_new=(path_prev[0]-1,)
            selection.select_path(path_new)

    def directive_down(self, button, treeview):
        """Callback appelée pour redescendre une directive
        """
        selection = treeview.get_selection()
        model, iter_down = selection.get_selected()
        line_index = model.get_path(iter_down)[0]

        if line_index < (len(model)-1):
            self.swap(line_index, line_index+1)
            # on sélectionne la ligne à sa nouvelle position
            path_prev=model.get_path(iter_down)
            path_new=(path_prev[0]+1,)
            selection.select_path(path_new)

    def button_pressed(self, treeview, event):
        """Callback appelée lors d'un clic de souris sur le treeview.
        On l'utilise pour capturer le double-clic.
        """
        # bouton gauche
        if event.button == 1:
            # double-clic
            if event.type == gtk.gdk._2BUTTON_PRESS:
                selection = treeview.get_selection()
                store, iter_sel = selection.get_selected()
                if iter_sel is not None:
                    path = store.get_path(iter_sel)
                    selected_dir = store[path][-1]
                    dir_type = fwobjects.DIRECTIVE_TYPE_DOWN

                    dir_dlg = DirectiveDialog(self.glade_file,
                                              self.model,
                                              selected_dir.get_source_zone(),
                                              selected_dir.get_dest_zone(),
                                              selected_dir.get_directive_type(),
                                              self,
                                              selected_dir, iter_sel)

                    dir_dlg.show_dialog()


class DirectiveDialog(Singleton):
    """ Classe qui gère l'affichage de l'édition de directives
    """

    def __init__(self, glade_file, directive_store,
                 zoneA, zoneB, directive_type, ctrl,
                 edited_dir = None, dir_iter = None):
        """
        glade : l'objet XML glade
        directive_store : le classeur de directives
        """
        self.store = directive_store
        self.glade_file = glade_file
        self.directive_attrs = 0
        self.directive_type = directive_type
        self.service_group = False
        self.src_inv = 0
        self.dest_inv = 0
        self.serv_inv = 0
        self.ctrl = ctrl
        if 'dialog' not in self.__dict__ :
            self.src_handler_id  = None
            self.dest_handler_id  = None
            self.glade = gtk.glade.XML(self.glade_file,"directive_editor", "editeur")
            self.dialog = self.glade.get_widget('directive_editor')
            # self.dialog.set_size_request(670,390)
            self.available_view = self.glade.get_widget('rule_edit_available_tree')
            # self.directive_view = self.glade.get_widget('rule_edit_rule_view')
            self.src_listview = self.glade.get_widget('rule_edit_src_listview')
            self.dest_listview = self.glade.get_widget('rule_edit_dest_listview')

            self.service_button = self.glade.get_widget('drop_service_button')
            self.action_menu = self.glade.get_widget('rule_edit_action_menu')
            self.extr_button = self.glade.get_widget('drop_extremite_button')
            self.port_nat_entry = self.glade.get_widget('port_nat_entry')
            self.range_button = self.glade.get_widget('drop_timerange_button')
            self.user_group_button = self.glade.get_widget('drop_user_group_button')

            self.ip_nat_label = self.glade.get_widget('ip_nat_label')
            self.port_nat_label = self.glade.get_widget('port_nat_label')

            self.src_zone_label = self.glade.get_widget('src_zone_label')
            self.dest_zone_label = self.glade.get_widget('dest_zone_label')

            self.log_toggle = self.glade.get_widget('rule_edit_log_button')
            self.optional_toggle = self.glade.get_widget('rule_edit_optional_button')
            self.mark_combo = self.glade.get_widget('mark_combo')
            self.mark_value = self.glade.get_widget('mark_value')
            self.mark_value.set_sensitive(False)
            self._init_extremites_columns()
            self._init_available_columns()
            self.tag_combo = self.glade.get_widget('tag_combo')
            self.tag_combo.set_sensitive(0)
            if len(library_store.tags.keys()) > 0:
                self.tag_combo.set_popdown_strings( library_store.tags.keys() )
            # self.tag_combo.set_use_arrows_always(True)
            # inversion des extremites
            self.src_inv_button = self.glade.get_widget('src_exclude_checkbutton')
            self.dest_inv_button  = self.glade.get_widget('dest_exclude_checkbutton')
            self.serv_inv_button = self.glade.get_widget('serv_exclude_checkbutton')
            # libelle
            self.libelle = self.glade.get_widget('libelle_entry')

            handlers = {'on_directive_editor_response' : self.validate_edit,
                        'on_rule_edit_log_button_toggled' : (self.toggled, fwobjects.DIRECTIVE_LOG ),
                        'on_rule_edit_optional_button_toggled' : (self.optional_toggled, fwobjects.DIRECTIVE_OPTIONAL),
                        'on_forward_activate' : (self.set_natzone_sensitive, fwobjects.ACTION_FORWARD, True),
                        'on_dnat_activate' : (self.set_natzone_sensitive, fwobjects.ACTION_DNAT, True),
                        'on_rule_edit_src_listview_button_press_event' : self.remove_extremite,
                        'on_rule_edit_src_listview_key_press_event' : self.remove_extremite2,
                        'on_rule_edit_dest_listview_button_press_event' : self.remove_extremite,
                        'on_rule_edit_dest_listview_key_press_event' : self.remove_extremite2,
                        'on_snat_activate' : (self.set_natzone_sensitive, fwobjects.ACTION_MASK, True),
                        # inversion des extremités
                        'on_src_exclude_checkbutton_toggled' : self.invert_src,
                        'on_dest_exclude_checkbutton_toggled' : self.invert_dest,
                        'on_serv_exclude_checkbutton_toggled' : self.invert_serv,
                        # combo pour les directive optionnelles
                        # 'on_rule_edit_select_tag_combo_changed' : self.tag_list
                        'on_mark_combo_changed':self.set_mark_value_sensitive,
                        'on_drop_timerange_button_button_press_event':self.range_clicked,
                        'on_drop_user_group_button_button_press_event':self.user_group_clicked,
                        }

            self.glade.signal_autoconnect(handlers)
            self.dialog.connect('delete_event', self.hide_and_flush)

            selection = self.available_view.get_selection()
            selection.connect('changed',self.store_selected)

        self._guess_zones(zoneA,zoneB)

        self.edited_dir = edited_dir
        self.dir_iter = dir_iter

        self._init_buttons_lf()
        self._init_drag_destinations()
        self.add_default_policy_item()

        self._init_directive_view()
        self._init_available_view()

        if self.edited_dir is not None:
            self._fill_dialog()
            # si la directive n'est pas éditable, on grise la vbox principale
            if not self.edited_dir.is_editable():
                self.glade.get_widget('hpaned3').set_sensitive(0)
            else:
                self.glade.get_widget('hpaned3').set_sensitive(1)
        else:
            self.glade.get_widget('hpaned3').set_sensitive(1)
            self.selected_service = None
            self.selected_range = None
            self.selected_user_group = None
            self.nat_extr = None


    def _fill_dialog(self):
        """Remplit la boîte de dialogue avec les données de la directive
        à éditer. (self.edited_dir)
        """
        # On remplit la liste des sources
        for extr in self.edited_dir.src_list:
            self.src_model.append([extr.name,extr.libelle, extr.zone.name])
        # On remplit la liste des destinations
        for extr in self.edited_dir.dest_list:
            self.dest_model.append([extr.name,extr.libelle, extr.zone.name])

        self.service_button.set_label(str(self.edited_dir.service))
        self.selected_service = self.edited_dir.service
        self.selected_range = self.edited_dir.time
        # récupération des groupes d'utilisateurs
        self.selected_user_group = self.edited_dir.user_group
        if self.edited_dir.user_group != None:
            self.user_group_button.set_label(str(self.edited_dir.user_group.name))
        # tag ead
        self.libelle.set_text(self.edited_dir.libelle)
        try:
            self.tag_combo.entry.set_text(self.edited_dir.get_tag())
        except:
            self.tag_combo.entry.set_text("")
        # récupération de la plage horaire
        if self.edited_dir.time != None:
            self.range_button.set_label(self.edited_dir.time.name)
        # récupération des options de marquage
        model = self.mark_combo.get_model()
        try:
            cur_iter = model.get_iter_root()
            while cur_iter:
                if model.get_value(cur_iter,0) == self.edited_dir.mark[0]:
                    self.mark_combo.set_active_iter(cur_iter)
                    self.mark_value.set_text(self.edited_dir.mark[1])
                    break
                cur_iter=model.iter_next(cur_iter)
        except:
            self.mark_combo.set_active_iter(model.get_iter_root())

        # récupération de l'inversion des extremité et du service
        self.src_inv_button.set_active(self.edited_dir.src_inv)
        self.dest_inv_button.set_active(self.edited_dir.dest_inv)
        self.serv_inv_button.set_active(self.edited_dir.serv_inv)

        if self.edited_dir.nat_extr is not None:
            self.extr_button.set_label(self.edited_dir.nat_extr.name)
            self.nat_extr = self.edited_dir.nat_extr
            # self.port_nat_entry.set_sensitive(True)
        else:
            self.nat_extr = None
            self.ip_nat_label.set_sensitive(False)
            self.extr_button.set_sensitive(False)

        if self.edited_dir.nat_port is not None:
            self.port_nat_entry.set_text(str(self.edited_dir.nat_port))
        if (self.edited_dir.action == fwobjects.ACTION_DNAT or self.edited_dir.action == fwobjects.ACTION_MASK):
            self.extr_button.set_sensitive(True)
            self.port_nat_label.set_sensitive(True)
            self.port_nat_entry.set_sensitive(True)
        else:
            self.extr_button.set_sensitive(False)
            self.port_nat_label.set_sensitive(False)
            self.port_nat_entry.set_sensitive(False)

        if self.edited_dir.action == fwobjects.ACTION_FORWARD:
            self.port_nat_label.set_sensitive(True)
            self.port_nat_entry.set_sensitive(True)


        # XXX UGLY !! Mais c'est pas très pratique les optionmenus ...
        item_index = {fwobjects.ACTION_DENY : 0,
                      fwobjects.ACTION_ALLOW : 0,
                      fwobjects.ACTION_FORWARD : 1,
                      fwobjects.ACTION_DNAT : 2,
                      fwobjects.ACTION_MASK : 3 }
        self.action_menu.set_history(item_index[self.edited_dir.action])
        self.action_type = self.edited_dir.action
        # il ne faut pas reprendre directive_attrs dans la directive éditée,
        # sinon, sa valeur est inversée lorsque les cases sont cochées dans _directive_toggle
        self.directive_attrs = 0
        self._directive_toggle()

    def _directive_toggle(self):
        """Coche les checkbuttons pour correspondre à la directive éditée
        """
        if self.edited_dir.is_logged():
            self.log_toggle.set_active(True)
        if self.edited_dir.is_marked():
            self.mark_value.set_sensitive(True)
        else:
            self.mark_value.set_sensitive(False)
        if self.edited_dir.is_optional():
            self.optional_toggle.set_active(True)
            self.tag_combo.set_sensitive(1)
        else:
            self.tag_combo.set_sensitive(0)

    def _guess_zones(self, zoneA, zoneB):
        """Permet de savoir quelle zone est la source et quelle source
        est la destination
        """
        if self.directive_type == fwobjects.DIRECTIVE_TYPE_UP:
            if zoneA < zoneB:
                self.src_zone, self.dest_zone = zoneA, zoneB
            else:
                self.src_zone, self.dest_zone = zoneB, zoneA
        else:
            if zoneA > zoneB:
                self.src_zone, self.dest_zone = zoneA, zoneB
            else:
                self.src_zone, self.dest_zone = zoneB, zoneA
        self.src_zone_label.set_label('<span foreground="#a0569a" weight="bold">%s</span>'% \
                                      self.src_zone.name)
        self.dest_zone_label.set_label('<span foreground="#a0569a" weight="bold">%s</span>'% \
                                       self.dest_zone.name)



    def _init_buttons_lf(self):
        """Initialise le L&F (Look and feel) des boutons 'drop' ...
        """
        # initialisation du bouton : 'drop a service'
        self.service_button.remove(self.service_button.get_children()[0])
        l = gtk.Label(_("drop a service"))
        l.set_use_markup(True)
        self.service_button.add(l)
        self.service_button.show_all()
        # initialisation du bouton : 'drop extremite'
        self.extr_button.remove(self.extr_button.get_children()[0])
        l = gtk.Label(_("drop extremite"))
        l.set_use_markup(True)
        self.extr_button.add(l)
        self.extr_button.show_all()
        # initialisation du bouton : 'drop user_group'
        self.user_group_button.remove(self.user_group_button.get_children()[0])
        l = gtk.Label(_("drop user_group"))
        l.set_use_markup(True)
        self.user_group_button.add(l)
        self.user_group_button.show_all()


    def _init_drag_destinations(self):
        """Initialise l'ensemble des destinations de DnD
        """
        self.service_button.connect("drag_data_received", self.receive_service)
        self.service_button.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
                                          gtk.DEST_DEFAULT_HIGHLIGHT |
                                          gtk.DEST_DEFAULT_DROP,
                                          [( "library/object", 0, 0 )],
                                          gtk.gdk.ACTION_COPY)

        self.extr_button.connect("drag_data_received", self.nat_button_receive_extremite)
        self.extr_button.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
                                       gtk.DEST_DEFAULT_HIGHLIGHT |
                                       gtk.DEST_DEFAULT_DROP,
                                       [( "library/object", 0, 0 )],
                                       gtk.gdk.ACTION_COPY)

        if self.src_handler_id is not None:
            self.src_listview.disconnect(self.src_handler_id)
        self.src_handler_id = self.src_listview.connect("drag_data_received", self.receive_extremite, self.src_zone.name)
        self.src_listview.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
                                        gtk.DEST_DEFAULT_HIGHLIGHT |
                                        gtk.DEST_DEFAULT_DROP,
                                        [( "library/object", 0, 0 )],
                                        gtk.gdk.ACTION_COPY)

        if self.dest_handler_id is not None:
            self.dest_listview.disconnect(self.dest_handler_id)
        self.dest_handler_id = self.dest_listview.connect("drag_data_received", self.receive_extremite, self.dest_zone.name)
        self.dest_listview.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
                                         gtk.DEST_DEFAULT_HIGHLIGHT |
                                         gtk.DEST_DEFAULT_DROP,
                                         [( "library/object", 0, 0 )],
                                         gtk.gdk.ACTION_COPY)


        self.range_button.connect("drag_data_received", self.receive_range)
        self.range_button.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
                                          gtk.DEST_DEFAULT_HIGHLIGHT |
                                          gtk.DEST_DEFAULT_DROP,
                                          [( "library/object", 0, 0 )],
                                          gtk.gdk.ACTION_COPY)

        self.user_group_button.connect("drag_data_received", self.receive_user_group)
        self.user_group_button.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
                                          gtk.DEST_DEFAULT_HIGHLIGHT |
                                          gtk.DEST_DEFAULT_DROP,
                                          [( "library/object", 0, 0 )],
                                          gtk.gdk.ACTION_COPY)



    def add_default_policy_item(self):
        """Ajoute l'action 'Autoriser' ou 'Interdire' suivant le type de règle
        """
        menu = self.action_menu.get_menu()
        # XXX FIXME : BEURK ! Mais comment changer le nom d'un item ?!!
        menu.remove(menu.get_children()[0])
        # dans le cas ou les zones sont de même niveau, on met toujours autoriser
        if (self.directive_type == fwobjects.DIRECTIVE_TYPE_UP) or (self.src_zone.level==self.dest_zone.level):
            item = gtk.MenuItem(_('allow'))
            self.action_type = fwobjects.ACTION_ALLOW
        else:
            item = gtk.MenuItem(_('deny'))
            self.action_type = fwobjects.ACTION_DENY
        item.connect('activate', self.set_natzone_sensitive, self.action_type,False)
        # on ajoute cet item en tête
        menu.prepend(item)
        self.action_menu.set_menu(menu)
        # on impose à l'action menu de le montrer
        self.action_menu.set_history(0)


    def set_natzone_sensitive(self, item, action_type, sensitive):
        """ Grise ou dégrise la zone concernant la zone de translation
        """
        # Cas particulier du redirect : ça va sur Localhost
        if action_type ==  fwobjects.ACTION_FORWARD:
            self.extr_button.set_sensitive(False)
            self.ip_nat_label.set_sensitive(False)
        else:
            self.extr_button.set_sensitive(sensitive)
            self.ip_nat_label.set_sensitive(sensitive)
        self.action_type = action_type
        self.port_nat_label.set_sensitive(sensitive)
        self.port_nat_entry.set_sensitive(sensitive)

    def set_mark_value_sensitive(self, *args):
        self.mark_value.set_sensitive(self.mark_combo.get_active_text() != "")

    def _init_directive_view(self):
        """ Initialise la vue et le modèle associés à la directive
        """
        self.src_listview.set_headers_visible(1)
        self.dest_listview.set_headers_visible(1)

        self.src_model = create_extremite_treemodel()
        self.dest_model = create_extremite_treemodel()

        self.src_listview.set_model(self.src_model)
        self.dest_listview.set_model(self.dest_model)


    def _init_extremites_columns(self):
        column_names = [_('name'), _('libelle'), _('zone')]

        col_index = 0
        for col_name in column_names:
            src_column = gtk.TreeViewColumn(col_name,
                                            gtk.CellRendererText(),
                                            text = col_index)

            dest_column = gtk.TreeViewColumn(col_name,
                                             gtk.CellRendererText(),
                                             text = col_index)
            self.src_listview.append_column(src_column)
            self.dest_listview.append_column(dest_column)

            col_index += 1


    def _init_available_view(self):
        """ Initialise la vue et le modèle associés à l'arbre des disponibilités
        """
        self.available_model = create_objects_treestore(self.src_zone,
                                                        self.dest_zone)
        self.available_view.set_model(self.available_model)
        self.available_view.connect_after('drag_data_get', self.send_callback)
        self.available_view.drag_source_set(gtk.gdk.BUTTON1_MASK,
                                            [( "library/object", 0, 0 )],
                                            gtk.gdk.ACTION_COPY)
        # init du D&D (Drag and drop) sur le bouton des timeranges pour les supprimer
        self.range_button.drag_source_set(gtk.gdk.BUTTON1_MASK,
                                            [( "library/object", 0, 0 )],
                                            gtk.gdk.ACTION_MOVE)
        self.range_button.connect("drag_end", self.drop_range)
        # init du D&D sur le bouton des user_groups pour les supprimer
        self.user_group_button.drag_source_set(gtk.gdk.BUTTON1_MASK,
                                            [( "library/object", 0, 0 )],
                                            gtk.gdk.ACTION_MOVE)
        self.user_group_button.connect("drag_end", self.drop_user_group)

    def _init_available_columns(self):
        """Définit les colonnes de l'arbre des objets disponible
        """
        column_names = [_("available objects")]
        col_index = 0
        for col_name in column_names:
            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn(col_name, renderer,
                                        text = col_index)
            self.available_view.append_column(column)
            col_index += 1


        # self.available_view.connect('button-press-event', self.store_selected)


    def store_selected(self, selection, *args):
        model, iter_sel = selection.get_selected()
        if iter_sel is not None:
            # iter_sel est nul si on replie un arbre
            path = model.get_path(iter_sel)
            self.selected_object = model[path][1]


    # XXX FIXME : pour les types, utiliser directement les types Mime plutôt
    # que de redéfinir get_type() sur la classe Node !
    # Comment faire pour utiliser des mimetypes différents suivant les noeuds ?
    def receive_extremite(self, widget, context, x, y, selection, targetType, time, zone_name):
        """Appelée lors d'un Drop d'une extremité
        """
        widget.emit_stop_by_name('drag_data_received')

        try:
            if self.selected_object.get_type() == "firewall/extremite":
                store = widget.get_model()
                added_extr = [row[0] for row in store]
                extr = self.selected_object

                if extr.zone.name != zone_name:
                    dlg = create_error_dialog(_("bad zone drop"))
                    dlg.show_all()
                    return True
                if extr.name not in added_extr :
                    store.append((extr.name, extr.libelle, extr.zone.name))
                else:
                    dlg = create_error_dialog(_("already added extremite"))
                    dlg.show_all()
                    return True

                self.test_check_invert(store,widget)

        except TypeError:
            pass


    def test_check_invert(self,store,widget):
        first_iter =  store.get_iter_first()
        if first_iter == None:
            if widget == self.dest_listview:
                self.glade.get_widget('dest_exclude_checkbutton').set_sensitive(1)
            else:
                self.glade.get_widget('src_exclude_checkbutton').set_sensitive(1)
        else:
            # on récupère l'extrémité correspondante
            nb_ip = len(library_store.extremites[store.get_value(first_iter,0)].ip_list)
            if nb_ip > 1 or store.iter_next(store.get_iter_first()) is not None:
                # on grise la case à cocher
                if widget == self.dest_listview:
                    self.glade.get_widget('dest_exclude_checkbutton').set_sensitive(0)
                    self.glade.get_widget('dest_exclude_checkbutton').set_active(0)
                else:
                    self.glade.get_widget('src_exclude_checkbutton').set_sensitive(0)
                    self.glade.get_widget('src_exclude_checkbutton').set_active(0)
            else:
                if widget == self.dest_listview:
                    self.glade.get_widget('dest_exclude_checkbutton').set_sensitive(1)
                else:
                    self.glade.get_widget('src_exclude_checkbutton').set_sensitive(1)


    def remove_extremite(self, treeview, event, *args):
        """ Appelée lors d'un click dans les listes de sources et destinations
        """
        if event.button == 1:
            # double-clic
            if event.type == gtk.gdk._2BUTTON_PRESS:
                selection = treeview.get_selection()
                store, iter_del = selection.get_selected()
                if iter_del is not None:
                    path = store.get_path(iter_del)
                    store.remove(iter_del)
                    self.test_check_invert(store,treeview)

    def remove_extremite2(self, treeview, event, *args):
        """ Appelée lors d'un click dans les listes de sources et destinations
        """
        if event.keyval == gtk.gdk.keyval_from_name("Delete"):
            selection = treeview.get_selection()
            store, iter_del = selection.get_selected()
            if iter_del is not None:
                path = store.get_path(iter_del)
                store.remove(iter_del)
                self.test_check_invert(store,treeview)


    def nat_button_receive_extremite(self, button, context, x,y, selection, targetType, time):
        """Callback appelée lorsque l'on fait reçoit l'évènement 'drag_data_received'
        sur le bouton 'drop_extremite'
        """
        # XXX FIXME : tester si l'extremité vient de la source ou de la dest !
        if self.selected_object.get_type() == "firewall/extremite":
            button.set_label(self.selected_object.name)
            self.nat_extr = self.selected_object



    def receive_service(self, button, context, x, y, selection, targetType,time):
        """Appelée lors d'un Drop dans le bouton 'drop service'
        """
        if self.selected_object.get_type() in ["firewall/service","firewall/group"]:
            if self.selected_object.get_type() == "firewall/group":
                self.glade.get_widget('serv_exclude_checkbutton').set_sensitive(0)
                self.glade.get_widget('serv_exclude_checkbutton').set_active(0)
            else:
                self.glade.get_widget('serv_exclude_checkbutton').set_sensitive(1)

            if self.selected_object.get_type() == "firewall/service":
                self.service_group = False
            else:
                self.service_group = True
            button.set_label(str(self.selected_object))
            self.selected_service = self.selected_object

    def drop_range(self, button, *args):
        """Appelé lors de la fin d'un d&d
        """
        if self.selected_range != None:
            button.set_label(_("drop timerange"))
            self.selected_range = None

    def range_clicked(self, button, event, *args):
        """Appelé lors de la fin d'un d&d
        """
        if event.type == gtk.gdk._2BUTTON_PRESS:
            if self.selected_range != None:
                button.set_label(_("drop timerange"))
                self.selected_range = None

    def drop_user_group(self, button, *args):
        """Appelé lors de la fin d'un d&d
        """
        if self.selected_user_group != None:
            button.set_label(_("drop user_group"))
            self.selected_user_goup = None

    def user_group_clicked(self, button, event, *args):
        """Appelé lors de la fin d'un d&d
        """
        if event.type == gtk.gdk._2BUTTON_PRESS or event.type == gtk.gdk.BUTTON_PRESS:
            if self.selected_user_group != None:
                button.set_label(_("drop user_group"))
                self.selected_user_group = None


    def receive_range(self, button, context, x, y, selection, targetType,time):
        """Appelée lors d'un Drop dans le bouton 'drop range'
        """
        if self.selected_object.get_type() == "firewall/range":
            button.set_label(self.selected_object.name)
            self.selected_range = self.selected_object

    def receive_user_group(self, button, context, x, y, selection, targetType,time):
        """Appelée lors d'un Drop dans le bouton 'drop user_group'
        """
        if self.selected_object.get_type() == "firewall/user_group":
            button.set_label(self.selected_object.name)
            self.selected_user_group = self.selected_object

    def send_callback(self, widget, context, selection, targetType, eventTime):
        if self.selected_object is not None:
            selection.set(selection.target, 8, self.selected_object.name)

    def toggled(self, button, flag):
        """ Callback appelée lorsque l'on (dé)coche une case de la fenêtre
        """
        self.directive_attrs = self.directive_attrs ^ flag

    def optional_toggled(self, checkbox, flag):
        """callback pour la case optionnelle qui permet d'afficher la combo
        """
        self.directive_attrs = self.directive_attrs ^ flag
        # dégrise les cases
        self.tag_combo.set_sensitive(checkbox.get_active())

    def tag_list(self, *args):
        """callback pour afficher la liste des tags de directive préexistants
        """
        if len(library_store.tags.keys()) > 0:
            self.tag_combo.set_popdown_strings( library_store.tags.keys() )

    def invert_src(self, button) :
        """Callback appelée lorsque l'on souhaite inverser une extrémité source
        """
        self.src_inv = button.get_active()

    def invert_dest(self, button) :
        """Callback appelée lorsque l'on souhaite inverser une extrémité destination
        """
        self.dest_inv = button.get_active()

    def invert_serv(self, button) :
        """Callback appelée lorsque l'on souhaite inverser une extrémité destination
        """
        self.serv_inv = button.get_active()

    def flush(self):
        """ Vide le dialogue d'édition
        """
        # On vide la liste de sources / dests
        self.src_listview.get_model().clear()
        self.dest_listview.get_model().clear()
        # On réinitialise les boutons de DnD
        self._init_buttons_lf()
        self.port_nat_entry.set_text("")
        self.libelle.set_text("")
        self.range_button.set_label(_("drop timerange"))
        self.selected_service = None
        self.selected_range = None
        self.selected_user_group = None
        self.nat_extr = None
        self.nat_port = None
        # On décoche tous les toggles
        self.ip_nat_label.set_sensitive(False)
        self.extr_button.set_sensitive(False)
        self.log_toggle.set_active(False)
        # self.mark_toggle.set_active(False)
        self.optional_toggle.set_active(False)
        # options de marquage
        model = self.mark_combo.get_model()
        self.mark_combo.set_active_iter(model.get_iter_root())
        self.mark_value.set_text("")
        # inversion des extremités et du service
        self.src_inv_button.set_active(0)
        self.dest_inv_button.set_active(0)
        self.serv_inv_button.set_active(0)

    def hide_and_flush(self, *args):
        """ Cache la fenêtre et la réinitialise
        """
        self.dialog.hide()
        self.flush()

        return True


    def get_extr_list(self, store):
        """ Récupère la liste des noms d'extrémités d'un modèle
        store : le modèle à traiter
        """
        return [row[0] for row in store]



    def validate_edit(self, dialog, response_code, *args):
        """ Callback appelée lorsque l'on a fini d'éditer une directive
        """

        # On ne tient compte que si l'on clique sur OK
        if response_code != gtk.RESPONSE_OK:
            return self.hide_and_flush()

        src_names = self.get_extr_list(self.src_listview.get_model())
        dest_names = self.get_extr_list(self.dest_listview.get_model())
        src_list = [library_store.extremites[name] for name in src_names]
        dest_list = [library_store.extremites[name] for name in dest_names]
        self.nat_port = self.port_nat_entry.get_text()
        self.opt_tag = self.tag_combo.entry.get_text()

        if self.libelle.get_text() :
            libelle = self.libelle.get_text()
        else :
            libelle = "pas de description"

        if not len(src_list):
            src_list.append(library_store.extremites[self.src_zone.name])
        if not len(dest_list):
              dest_list.append(library_store.extremites[self.dest_zone.name])

        mark_operator = self.mark_combo.get_active_text()
        if mark_operator != "":
            mark = (mark_operator, self.mark_value.get_text())
        else:
            mark=None

        try:
            d = fwobjects.Directive(src_list, dest_list, self.selected_service, self.action_type,self.directive_attrs,
                                    nat_port = self.nat_port, nat_extr = self.nat_extr,
                                    is_group = self.service_group,
                                    src_inv = self.src_inv,
                                    dest_inv = self.dest_inv,
                                    serv_inv = self.serv_inv,
                                    libelle = libelle,
                                    time = self.selected_range,
                                    mark = mark,
                                    user_group = self.selected_user_group)

            if self.glade.get_widget('rule_edit_optional_button').get_active() == 1:
                d.set_tag(self.opt_tag)

            # on met à jour les services ,extremites et plages horaires pour leur signaler qu'ils sont utilisés
            self.selected_service.used += 1
            for extr in src_list:
                extr.used += 1
            for extr in dest_list:
                extr.used += 1
            if self.selected_range != None:
                self.selected_range.used += 1
            if self.selected_user_group != None:
                self.selected_user_group.used += 1

            if self.edited_dir is not None:
                # c'est une directive éditée (pas une nouvelle directive)

                # on libère les anciens services, extremités et plages horaires
                self.edited_dir.service.used -= 1
                for extr in self.edited_dir.src_list:
                    extr.used -= 1
                for extr in self.edited_dir.dest_list:
                    extr.used -= 1
                if self.edited_dir.time != None:
                    self.edited_dir.time.used -= 1
                if self.selected_user_group != None:
                    self.selected_user_group.used -= 1

                # on met à jour la directive
                self.edited_dir.src_list = d.src_list
                self.edited_dir.dest_list = d.dest_list
                self.edited_dir.service = d.service
                self.edited_dir.action = d.action
                self.edited_dir.attrs = d.attrs
                #self.edited_dir.priority = d.priority
                self.edited_dir.nat_extr = d.nat_extr
                self.edited_dir.nat_port = d.nat_port
                self.edited_dir.service_group = d.service_group
                self.edited_dir.libelle = d.libelle
                self.edited_dir.time = d.time
                self.edited_dir.user_group = d.user_group
                self.edited_dir.mark = d.mark
                # inversion
                self.edited_dir.src_inv = d.src_inv
                self.edited_dir.dest_inv = d.dest_inv
                self.edited_dir.serv_inv = d.serv_inv
                # tag optionnel
                if self.glade.get_widget('rule_edit_optional_button').get_active() == 1:
                    if not self.opt_tag in library_store.tags.keys():
                        library_store.tags[str(self.opt_tag)] = (False,False)
                        self.tag_combo.set_popdown_strings(library_store.tags.keys())
                    self.edited_dir.set_tag(self.opt_tag)
                col_index = 0
                for val in self.edited_dir.as_list():
                    # self.store.notify_observers()
                    self.ctrl.gtk_model.set(self.dir_iter, col_index, val)
                    col_index += 1

                return self.hide_and_flush()
            else:
                # nouvelle directive
                self.store.add_directive(d)
                self.ctrl.gtk_model.append(d.as_list())
                # tag optionnel
                if self.glade.get_widget('rule_edit_optional_button').get_active() == 1:
                    if not self.opt_tag in library_store.tags.keys():
                        library_store.tags[str(self.opt_tag)] = (False,False)
                        self.tag_combo.set_popdown_strings(library_store.tags.keys())
                dlg = create_info_dialog(_('directive created'),self.dialog)
                dlg.show_all()

        except (BadPortError, AssertionError, TagError), e:
            dlg = create_error_dialog(str(e))
            dlg.show_all()
            return


        self.flush()

    def show_dialog(self):
        """ Affiche la fenêtre de dialogue d'édition de directive
        """
        self.dialog.show_all()

    def assert_directive_valid(self, directive):
        """ Teste si la directive est valide
        """
        pass


