# -*- coding: UTF-8 -*-

"""
Module concernant les options du modèle
"""

import gtk, gtk.glade
import gobject
from era.noyau.pool import library_store

class LibelleOptionsDialog:
    """Gère l'activation et la désactivation des options.
    """

    def __init__(self, glade_file, serv_edit=None):
        """
        glade : l'objet XML glade
        """
        self.glade_file = glade_file
        self.glade = gtk.glade.XML(self.glade_file,"options_dialog", "editeur")
        self.dlg = self.glade.get_widget("options_dialog")

        self.tree_view = self.glade.get_widget('options_treeview')

        handlers = {'on_options_dialog_response' : self.close,
                    # 'on_extremite_listview_dialog_delete_event' : self.stay_alive,
                    'on_options_treeview_button_press_event':self.get_selected_clicked,
                    'on_activate_button_clicked':self.get_selected,
                   }
        self.glade.signal_autoconnect(handlers)

        store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)

        renderer = gtk.CellRendererText()
        tog_renderer = gtk.CellRendererText()

        column = gtk.TreeViewColumn(_('option label'), renderer, text= 0)
        column.set_expand(True)
        column2 = gtk.TreeViewColumn(_('option active'), tog_renderer, text=2)

        self.tree_view.set_model(store)
        self.tree_view.append_column(column)
        self.tree_view.append_column(column2)

        for option in library_store.options.keys():
            store.append([option, library_store.options[option], self.bool_to_string(library_store.options[option])])


    def bool_to_string(self, bool):
        if bool:
            return _("option_active")
        else:
            return _("option_inactive")
        
    def stay_alive(self, *args):
        self.close()
        return True

    def close(self, button, code, *args):
        if code == gtk.RESPONSE_OK:
            # mise à jour de library_store
            self.tree_view.get_model().foreach(self.save_options,[])
        self.dlg.hide()

    def save_options(self, model, path, current_iter, data):
        option = str(model.get_value(current_iter,0))
        actif = model.get_value(current_iter,1)
        library_store.options[option]=int(actif)

    def get_selected_clicked(self,treeview,event,*args):
        """Supprime la directive selectionnée par click
        """
        if event.type == gtk.gdk._2BUTTON_PRESS:
            self.get_selected(treeview)
        
    def get_selected(self,treeview,*args):
        """Supprime la directive selectionnée
        """
        selection = self.tree_view.get_selection()
        store, iter_sel  = selection.get_selected()

        if iter_sel is not None:
            # tag=store.get_value(iter_sel,0)
            actif=store.get_value(iter_sel,1)
            if actif:
                store.set_value(iter_sel,1,False)
                store.set_value(iter_sel,2,self.bool_to_string(False))
            else:
                store.set_value(iter_sel,1,True)
                store.set_value(iter_sel,2, self.bool_to_string(True))

    def show_dialog(self):
        self.dlg.show_all()
