# -*- coding: UTF-8 -*-
"""Gestion des fenêtres de classes de qos
"""

import gtk, gtk.glade
import gobject

from era.noyau.fwobjects import QosClass
from era.noyau.dpatterns import Singleton
from gtk_models import create_qos_treestore
from ihm_utils import create_error_dialog, create_yes_no_dialog
from era.noyau.pool import library_store

class QosListviewDialog(Singleton):

    def __init__(self, glade_file):

        self.glade_file = glade_file
        
        if 'dlg' not in self.__dict__:
            self.glade = gtk.glade.XML(self.glade_file,"qos_listview_dialog", "editeur")
            self.dlg = self.glade.get_widget('qos_listview_dialog')
            self.treeview =  self.glade.get_widget('qos_listview')
            self.treeview.set_headers_visible(1)
            self.prop_label =  self.glade.get_widget('qos_properties_label')
            self.init_treeview_column()
            selection = self.treeview.get_selection()
            selection.set_mode(gtk.SELECTION_SINGLE)
            selection.connect('changed',self.item_selected)
            handlers = {'on_qos_listview_dialog_response' : self.close,
                        'on_qos_list_close_button_press_event': self.close,
                        'on_qos_listview_dialog_delete_event' : self.stay_alive,
                        'on_edit_qos_button_clicked' : self.edit_qos,
                        'on_del_qos_button_clicked' : self.del_qos
                        }
            self.glade.signal_autoconnect(handlers)

        self.init_model()

    def init_model(self):
        """Initialise le modèle associé à l'arbre des qoss
        """
        self.treemodel = create_qos_treestore()
        self.treeview.set_model(self.treemodel)

    def init_treeview_column(self):
        column = gtk.TreeViewColumn(_('qos'),
                                    gtk.CellRendererText(),
                                    text = 0)
        self.treeview.append_column(column)

    def item_selected(self,selection, *args):
        """Callback appelée lorsqu'on clique sur un qos dans l'arbre
        """
        (store, itersel) = selection.get_selected()
        if itersel is None:
            self.selected_node = None
            return
        
        self.selected_node = store.get_value(itersel,1)
        # On met à jour les propriétés
        self.update_properties()

    def update_properties(self):
        """Mets à jour la partie 'propriétés' de la fenêtre de dialogue
        (partie droite)
        """
        # on vérifie si l'objet peut-être modifié
        if self.selected_node is not None:
            if self.selected_node.is_editable():
                self.glade.get_widget('del_qos_button').set_sensitive(1)
                self.glade.get_widget('edit_qos_button').set_sensitive(1)
            else:
                self.glade.get_widget('del_qos_button').set_sensitive(0)
                self.glade.get_widget('edit_qos_button').set_sensitive(0)
            self.prop_label.set_text(str(self.selected_node))
        else:
            self.glade.get_widget('del_qos_button').set_sensitive(0)
            self.glade.get_widget('edit_qos_button').set_sensitive(0)
            self.prop_label.set_text("")
            
## FIXME edition, suppresion de classe de qos à faire
    def edit_qos(self,*args):
        """Ouvre la fenetre d'edition de la classe de qos
        """
        if self.selected_node:
            editdlg=QosDialog(self.glade_file, self.selected_node)
            editdlg.show_dialog()
                
    def del_qos(self,*args):
        """supprime une  classe de qos de la bibliothèque
        """
        # FIXME
        pass
#         if self.selected_node:
#             # on vérifie si l'objet est encore utilisé
#             if self.selected_node.used > 0:
#                 dial_error = create_error_dialog(_("object still used"),self.dlg)
#                 dial_error.show()
#             else:
#                 # dialogue de confirmation
#                 dial_conf = create_yes_no_dialog(_("remove confirmation"))
#                 # dial_conf=gtk.MessageDialog(parent=self.dlg,buttons=gtk.BUTTONS_YES_NO)
#                 # dial_conf.label.set_text(
#                 dial_conf.set_default_response(gtk.RESPONSE_NO)
#                 ret = dial_conf.run()
#                 dial_conf.destroy()
#                 # on teste le retour du dialogue
#                 if ret == gtk.RESPONSE_YES:
#                     selection = self.treeview.get_selection()
#                     model, iter_sel = selection.get_selected()
                    
#                     if self.selected_node.get_type() == "firewall/group":
#                         # suppression d'un groupe
#                         # mise à jour des qoss correspondants
#                         for qos in self.selected_node.qoss:
#                             qos.used -= 1
#                         library_store.groups.remove(self.selected_node)
#                     else:
#                         # suppresion d'un qos
#                         library_store.qoss.remove(self.selected_node)

#                     self.treemodel.remove(iter_sel)
                
    def show_dialog(self, *args):
        self.treeview.set_size_request(250,300)
        self.dlg.show_all()

    def stay_alive(self, *args):
        self.close()
        return True

    def close(self, *args):
        self.dlg.hide()

class QosDialog(Singleton):
    """Création et édition de classes de qos    """
    # FIXME : édition et création d'une classe de qos -- la création peut etre accessible depuis la fenetre de dialogue elle-mme
    def __init__(self, glade_file, qos_edit=None):
        """
        glade : l'objet XML glade
        """

        self.glade_file = glade_file
        self.glade = gtk.glade.XML(self.glade_file,"qos_dialog", "editeur")

        if 'dlg' not in self.__dict__:
            self.dlg = self.glade.get_widget('qos_dialog')
            self.name_entry = self.glade.get_widget('qos_name_entry')
            self.libelle_entry = self.glade.get_widget('qos_libelle_entry')
            self.bandwidth_entry = self.glade.get_widget('qos_bandwidth_entry')
      
            handlers = {
                # 'on_qos_dialog_response' : self.validate_edit,
                'on_qos_editor_validate_button_press_event' : self.validate_edit,
                # FIXME : bouton de creation d'une nouvelle classe de qos
#                 'on_qos_dialog_delete_event' : self.stay_alive,
                }
        
            self.glade.signal_autoconnect(handlers)
        # la classe de qos à éditer
        self.qos = qos_edit
        if qos_edit != None:
            self.name_entry.set_text(qos_edit.name)
            self.name_entry.set_sensitive(0)
            self.bandwidth_entry.set_text(qos_edit.bandwidth)
            self.libelle_entry.set_text(qos_edit.libelle)

#     def stay_alive(self,*args):
#         self.hide_and_flush(*args)
#         return True
    
    def hide_and_flush(self, *args):
        self.dlg.hide()
        self.flush()

    def flush(self):
        """Réinitialise l'ensemble des champs de la fenêtre d'édition de service
        """
        self.name_entry.set_text("")
        self.libelle_entry.set_text("")
        self.qos = None
        self.name_entry.set_sensitive(1)
        self.bandwidth_entry.set_text("")

    def validate_edit(self, dialog, response_code):
        """Callback appelée lorsqu'on valide l'édition d'une  classe de qos.
        """
        try:
            name = self.name_entry.get_text()
            bandwidth = self.bandwidth_entry.get_text()
            libelle = self.libelle_entry.get_text()
            if not name:
                raise EmptyNameError
            # creation d'un objet de qos
            if self.qos == None:
                #vérification de la non existence du nom (nouveau service)
                for qos in library_store.qos:
                    if qos.name == name:
                        dlg_er = create_error_dialog(_('existing qos class'),self.dlg)
                        dlg_er.show()
                        return
                new_qos = QosClass(name, bandwidth, libelle=libelle)
                library_store.add_qos(new_qos)
                self.flush()
            else:
                # edition d'un objet de qos
                self.qos.bandwidth = bandwidth
                self.qos.libelle=libelle
                self.hide_and_flush()
        except EmptyNameError:
            dlg = create_error_dialog(_('empty qos class'))
            dlg.show_all()

    def show_dialog(self):
        self.dlg.show_all()
