# -*- coding: UTF-8 -*-

"""
Module concernant les groupes et utilisateurs (NuFW)
"""

import gtk, gtk.glade
import gobject
import re, time
from era.noyau.pool import library_store
from era.noyau.fwobjects import UserGroup
from ihm_utils import *


class UserGroupDialog:
    """Gère les utilisateurs
    """


    def __init__(self, glade_file):
        """
        glade : l'objet XML glade
        """
        self.glade_file = glade_file
        self.glade = gtk.glade.XML(self.glade_file,"user_group_dialog", "editeur")
        self.dlg = self.glade.get_widget("user_group_dialog")

        self.tree_view = self.glade.get_widget('user_group_treeview')

        handlers = {'on_user_group_dialog_response' : self.close,
                    'on_user_group_treeview_button_press_event':self.get_selected_clicked,
                    'on_add_user_group_button_clicked':self.add_user_group,
                    'on_button_edit_user_group_clicked':self.get_selected,
                    'on_button_del_user_group_clicked':self.del_user_group,
                    'on_button_valid_user_group_clicked':self.validate_edit,
                   }
        self.glade.signal_autoconnect(handlers)

        store = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)

        renderer = gtk.CellRendererText()

        column = gtk.TreeViewColumn(_('user_group description'), renderer, text=1)
        column.set_expand(True)
        self.tree_view.set_model(store)
        self.tree_view.append_column(column)
        tree_sel = self.tree_view.get_selection()
        tree_sel.connect('changed', self.selected_user_group_changed)

        for user_group_id, user_group in library_store.user_groups.items():
            store.append([user_group, user_group.name])

    def close(self, button, code, *args):
        self.dlg.hide()

    def save_tags(self, model, path, current_iter, data):
        user_group = model.get_value(current_iter, 0)
        library_store.user_groups[user_group.id] = user_group

    def selected_user_group_changed(self, selection, *args):
        # récupération de l'objet sélectionnée
        store, iter_sel = selection.get_selected()
        if iter_sel is not None:
            selected_user_group = store.get_value(iter_sel,0)
            if selected_user_group is not None:
                self.glade.get_widget('button_del_user_group').set_sensitive(True)
                self.selected_user_group = selected_user_group
                # texte de description du groupe
                self.glade.get_widget('user_group_name').set_text(selected_user_group.name)
                self.glade.get_widget('user_group_id').set_text(selected_user_group.id)

    def get_selected_clicked(self,widget,event,*args):
        """récupère le double click dans le treeview
        """
        # édition si double click
        if event.type == gtk.gdk._2BUTTON_PRESS:
            self.get_selected(widget)
        
    def get_selected(self,widget,*args):
        """édite l'objet sélectionné
        """
        store, iter_sel = self.tree_view.get_selection().get_selected()

        if self.selected_user_group is not None:
            # édition de la plage
            dial_edit = UserGroupEditDialog(self.glade_file, self.tree_view.get_model(),iter_sel,self.selected_user_group)
            dial_edit.show_dialog()

    def add_user_group(self,*args):
        """édite l'objet sélectionné
        """
        model = self.tree_view.get_model()
        iter_add  = model.append()
        new_user_group = UserGroup("-1",_('New'))
        self.selected_user_group = new_user_group
        library_store.user_groups[self.selected_user_group.id]=self.selected_user_group
        model.set_value(iter_add,0,new_user_group)
        model.set_value(iter_add,1,new_user_group.name)
        self.glade.get_widget('user_group_name').set_text(new_user_group.name)
        self.glade.get_widget('user_group_id').set_text(new_user_group.id)
        # on sélectionne le nouveau groupe
        sel = self.tree_view.get_selection()
        sel.select_iter(iter_add)
        self.glade.get_widget('user_group_name').grab_focus()

    def del_user_group(self, *args):
        """supprime l'objet sélectionné
        """
        selection = self.tree_view.get_selection()
        store, iter_sel  = selection.get_selected()

        if iter_sel is not None:
            user_group=store.get_value(iter_sel,0)
            if self.selected_user_group.used > 0:
                # plage utilisée par des directives 
                dial_error = create_error_dialog(_("object still used"),self.dlg)
                dial_error.show()
            else:
                # suppression
                store.remove(iter_sel)
                del(library_store.user_groups[self.selected_user_group.id])
                self.glade.get_widget('button_del_user_group').set_sensitive(False)
                self.glade.get_widget('user_group_name').set_text('')
                self.glade.get_widget('user_group_id').set_text('')

    def validate_edit(self, *args):
        """valide les modifications sur un groupe
        """
        selection = self.tree_view.get_selection()
        model, iter_sel  = selection.get_selected()
        name = self.glade.get_widget('user_group_name').get_text()
        gid = self.glade.get_widget('user_group_id').get_text()
        old_id = self.selected_user_group.id
        if old_id != gid:
            if gid in library_store.user_groups.keys():
                # id existant : on interdit l'écrasement
                msg = _('identifier already exists')  + ' : %s' % gid
                dial_alerte = create_error_dialog(msg)
                dial_alerte.show()
                self.glade.get_widget('user_group_id').set_text(old_id)
                self.glade.get_widget('user_group_id').grab_focus()
                return False
            else:
                del(library_store.user_groups[old_id])
        self.selected_user_group.name = name
        self.selected_user_group.id = gid
        library_store.user_groups[gid]=self.selected_user_group
        model.set_value(iter_sel,0,self.selected_user_group)
        model.set_value(iter_sel,1,self.selected_user_group.name)
 
    def show_dialog(self):
        self.dlg.show_all()

#class RangeEditDialog:
#    """édite une plage horaire
#    """
#
#    def __init__(self, glade_file, model, iter_sel=None, range_edit=None):
#        """
#        glade : l'objet XML glade
#        """
#
#        self.days = {'Mon': _("Monday"),
#                     'Tue': _("Tuesday"),
#                     'Wed': _("Wednesday"),
#                     'Thu': _("Thursday"),
#                     'Fri': _("Friday"),
#                     'Sat': _("Saturday"),
#                     'Sun': _("Sunday")
#                     }
#
#        self.glade_file = glade_file
#        self.glade = gtk.glade.XML(self.glade_file,"range_edit_dialog", "editeur")
#        self.dlg = self.glade.get_widget("range_edit_dialog")
#        self.glade.get_widget("vbox_edit").set_sensitive(True)
#        if range_edit != None:
#            if not range_edit.is_editable():
#                self.glade.get_widget("vbox_edit").set_sensitive(False)
#        
#        handlers = {'on_range_edit_dialog_response' : self.close,
#                    'on_range_edit_dialog_delete_event' : self.close,
#                    'on_button_add_day_clicked':self.add_day,
#                    'on_button_remove_day_clicked':self.remove_day,
#                   }
#        self.glade.signal_autoconnect(handlers)
#        
#        self.order_days=['Mon','Tue','Wed','Thu','Fri','Sat','Sun']
#        self.model = model
#        self.iter_sel = iter_sel
#        self.combo_box = self.glade.get_widget('combo_days')
#        # remplissage du combo
#        self.combo_model = gtk.ListStore(gobject.TYPE_STRING,gobject.TYPE_STRING)
#        cell = gtk.CellRendererText()
#        self.combo_box.pack_start(cell, True)
#        self.combo_box.add_attribute(cell, 'text', 1)
#        self.combo_box.set_model(self.combo_model)
#        # récupération des paramètres de la plage horaire
#        self.set_date('entry_timestart', ':')
#        self.set_date('entry_timestop', ':')
#        self.set_date('entry_datestart', '::')
#        self.set_date('entry_datestop', '::')
#        self.selected_days = []
#        self.timerange = range_edit
#        self.update_data()
#
#    def update_data(self):
#        """remplit les champs du dialogue
#        """
#        self.combo_model.clear()
#        for day in self.order_days:
#            iter = self.combo_model.append()
#            self.combo_model.set_value(iter, 0, day)
#            self.combo_model.set_value(iter, 1, self.days[day])
#        iter_root = self.combo_model.get_iter_root()
#        self.combo_box.set_active_iter(iter_root)
#        if self.timerange != None:
#            if self.timerange.days != "":
#                self.selected_days = self.timerange.days.split(',')
#            self.set_date('entry_timestart', self.timerange.timestart)
#            self.set_date('entry_timestop', self.timerange.timestop)
#            self.set_date('entry_datestart', self.timerange.datestart)
#            self.set_date('entry_datestop', self.timerange.datestop)
#            translated_days = [self.days[day] for day in self.selected_days]
#            buff = self.glade.get_widget('textview_days').get_buffer()
#            buff.set_text(", ".join(translated_days))
#            self.glade.get_widget('entry_name').set_text(self.timerange.name)
#
#    def set_date(self, basename, data):
#        date_parts=data.split(':')
#        if date_parts != ['']:
#            for i in range(len(date_parts)):
#                widget = '%s%d'%(basename, i+1)
#                self.glade.get_widget(widget).set_text(date_parts[i])
#
#    def get_date(self, basename):
#        date_parts = []
#        for i in range(1,4):
#            widget = 'entry_%s%d' % (basename, i)
#            widget = self.glade.get_widget(widget)
#            if widget != None:
#                date_parts.append(widget.get_text())
#                
#        res = ':'.join(date_parts)
#        if res == '::' or res == ':':
#            res = ''
#        return res
#
#    def add_day(self, button):
#        """ajoute le jour sélectionné
#        """
#        self.combo_box = self.glade.get_widget('combo_days')
#        iter_active = self.combo_box.get_active_iter()
#        if iter_active != None:
#            add_day = self.combo_model.get_value(iter_active, 0)
#            if add_day not in self.selected_days:
#                # on reconstruit pour garder l'ordre
#                self.selected_days.append(add_day)
#                new_selected_days = []
#                for ordered_day in self.order_days:
#                    if ordered_day in self.selected_days:
#                        new_selected_days.append(ordered_day)
#                self.selected_days = new_selected_days
#                translated_days = [self.days[day] for day in self.selected_days]
#                buff = self.glade.get_widget('textview_days').get_buffer()
#                buff.set_text(", ".join(translated_days))
#
#    def remove_day(self, button):
#        """supprime le jour sélectionné
#        """
#        self.combo_box = self.glade.get_widget('combo_days')
#        iter_active = self.combo_box.get_active_iter()
#        if iter_active != None:
#            del_day = self.combo_model.get_value(iter_active, 0)
#            if del_day in self.selected_days:
#                self.selected_days.remove(del_day)
#                translated_days = [self.days[day] for day in self.selected_days]
#                buff = self.glade.get_widget('textview_days').get_buffer()
#                buff.set_text(", ".join(translated_days))
#
#    def show_dialog(self):
#        self.dlg.show_all()
#    
#    def save_range(self):
#        """sauvegarde les modifs
#        """
#        name = self.glade.get_widget("entry_name").get_text()
#        timestart = self.get_date('timestart')
#        timestop = self.get_date('timestop')
#        datestart = self.get_date('datestart')
#        datestop = self.get_date('datestop')
#        list_days = ','.join(self.selected_days)
#        # vérification des données:
#        return self.valid_saisie(name, timestart, timestop, datestart, datestop, list_days)
#
#    def valid_saisie(self, name, timestart, timestop, datestart, datestop, list_days):
#        errors = []
#        nbdates = nbhours = 0
#        msg = ""
#        if name == '':
#            msg = _("missing field name")
#        else:
#            for date in (datestart, datestop):
#                if date != '':
#                    if re.match('\d\d\d\d:\d+:\d+',date) == None:
#                        errors.append(date)
#                else:
#                    nbdates += 1
#            for data in (timestart, timestop):
#                if data != '':
#                    # format général
#                    if re.match('\d+:\d+',data) == None:
#                        errors.append(data)
#                else:
#                    nbhours += 1
#
#            if nbdates == 2 and nbhours == 2 and self.selected_days == []:
#                msg = _('missing fields')
#            elif errors != []:
#                msg = _('invalid format ') + ','.join(errors)
#            elif nbdates == 1 or nbhours == 1:
#                msg = _('start and stop needed')
#            else:
#                if nbhours == 0:
#                    # validité de la plage horaire
#                    try:
#                        t1 = time.strptime('%s:%s' % tuple(timestart.split(':')) ,'%H:%M')
#                        t2 = time.strptime('%s:%s' % tuple(timestop.split(':')) ,'%H:%M')
#                        # assert t1 < t2
#                    except AssertionError, ValueError:
#                        msg = _('invalid timerange')
#                if nbdates == 0:
#                    # validité de la plage par date
#                    try:
#                        t1 = time.strptime('%s:%s:%s' % tuple(datestart.split(':')) ,'%Y:%m:%d')
#                        t2 = time.strptime('%s:%s:%s' % tuple(datestop.split(':')) ,'%Y:%m:%d')
#                        assert t1 < t2
#                    except AssertionError, ValueError:
#                        msg = _('invalid timerange')
#
#        if msg != "":
#            dial_err = create_error_dialog(msg, self.dlg)
#            dial_err.show()
#            return False
#        # sauvegarde de l'objet
#        dico_range = dict(name=name,
#                          timestart=timestart,
#                          timestop=timestop,
#                          datestart=datestart,
#                          datestop=datestop,
#                          days=list_days)
#        library_store.ranges[name] = Range(dico_range)
#        # update du treeview
#        self.model.set_value(self.iter_sel,0,library_store.ranges[name])
#        self.model.set_value(self.iter_sel,1,library_store.ranges[name].name)
#        return True
#
#    def close(self, button, code, *args):
#        if code == gtk.RESPONSE_OK:
#            # sauvegarde du range
#            res=self.save_range()
#            if res == False:
#                return True
#        self.dlg.hide()
#        return True
#
#
