#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Tests de l'importation depuis l'ancien format

# FIXME --  

- test de tous les preceding-sibling :
  service : présent une seule fois
  etc...

"""

# parsing xsl
import libxml2
import libxslt
# tests simplifiés avec twisted domhelpers
from twisted.web import microdom
from twisted.web import domhelpers

XSL_FILE = "import.xsl"


def transform(xml_source):
    """traite le xsl et renvoie le xml en string
    """
    doc = libxml2.parseDoc(xml_source)

    styledoc = libxml2.parseFile(XSL_FILE)
    style = libxslt.parseStylesheetDoc(styledoc)

    result = style.applyStylesheet(doc, None)

    return result.serialize()


XML_FIREWALL = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
</flux>
</bastion>"""

def test_firewall():
    """
    le nom du bastion apparait bien
    >>> result = transform(XML_FIREWALL)
    >>> document = microdom.parseString(result)
    >>> racine = document.documentElement
    >>> racine.nodeName
    'firewall'
    >>> racine.hasAttribute("name")
    True
    >>> racine.getAttribute("name")
    'pare-feu-3cartes'
    """

def test_flux():
    """tests d'attributs zoneA et zoneB dans un flux
    """

XML_ZONES = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<!-- ===================================================================== -->
<zone niveau = "99" ip = "127.0.0.1" interface = "lo" network = "0.0.0.0" mask = "255.255.255.255">bastion</zone>
<zone niveau = "50" ip = "%%adresse_ip_eth1%%" interface = "eth1" network = "%%adresse_network_eth1%%" mask = "%%adresse_netmask_eth1%%">%%nom_machine_eth1%%</zone>
<zone niveau = "40" ip = "%%adresse_ip_eth2%%" interface = "eth2" network = "%%adresse_network_eth2%%" mask = "%%adresse_netmask_eth2%%">%%nom_machine_eth2%%</zone>
<zone niveau = "10" ip = "%%adresse_ip_eth0%%" interface = "%%interface_gw%%" network = "%%adresse_network_eth0%%" mask = "%%adresse_netmask_eth0%%">exterieur</zone>
<!-- ===================================================================== -->
</bastion>"""

def test_zones():

    """
    Les zones apparaissent bien
    >>> result = transform(XML_ZONES)
    >>> document = microdom.parseString(result)
    >>> zone = domhelpers.findNodesNamed(document, 'zone')[0]
    >>> zone
    Element('zone', attributes=InsensitiveDict({'name': 'bastion', 'level': '99', 'ip': '127.0.0.1', 'netmask': '255.255.255.255', 'interface': 'lo', 'network': '0.0.0.0'}), filename='<xmlfile />', markpos=(2, 148))
    >>>
    
    """

XML_OPTIONNEL1 = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<pont id = "connexion_dns" ordre = "130" libelle = "connexions au dns" optionnel="oui">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<port>53</port>
	</service> 
</pont>    
</montant>
</flux>
</bastion>"""

XML_OPTIONNEL2 = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<pont id = "connexion_dns" ordre = "130" libelle = "connexions au dns" optionnel="non">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<port>53</port>
	</service> 
</pont>    
</montant>
</flux>
</bastion>"""

def test_optionnel():
    """
    l'attribut renseignant les règles optionnelles est présent
    une règle optionnelle et une règle non optionnelle
    >>> result = transform(XML_OPTIONNEL1)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.hasAttribute("attrs")
    True
    >>> directive.getAttribute("attrs")
    '1'

    règle non optionnelle
    >>> result = transform(XML_OPTIONNEL2)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.hasAttribute("attrs")
    True
    >>> directive.getAttribute("attrs")
    '0'
    """

XML_SERVICE = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<pont id = "connexion_dns" ordre = "130" libelle = "connexions au dns">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<port>53</port>
	</service> 
</pont>    
</montant>
</flux>
</bastion>"""

XML_SERVICE2 = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<pont id = "connexion_dns" ordre = "130" libelle = "connexions au dns">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<plage haute="53" basse="43"/>
	</service> 
</pont>    
</montant>
</flux>
</bastion>"""

def test_service():
    """
    Le service associé à la règle apparait bien
    >>> result = transform(XML_SERVICE)
    >>> document = microdom.parseString(result)
    >>> services = domhelpers.findNodesNamed(document, "service")
    >>> dns = services[0]
    >>> dns.getAttribute("name")
    'dns'
    >>> dns.getAttribute("ports")
    '53'

    Test sur une plage de ports
    >>> 
    >>> result = transform(XML_SERVICE2)
    >>> document = microdom.parseString(result)
    >>> services = domhelpers.findNodesNamed(document, "service")
    >>> dns = services[0]
    >>> dns.getAttribute("ports")
    '43-53'
    >>> 

    """

def test_service_translation():
    """# FIXME --  La translation est aussi un service.
    """


XML_EXTREMITE = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<pont id = "connexion_dns" ordre = "130" libelle = "connexions au dns" optionnel="non">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<port>53</port>
	</service> 
</pont>    
</montant>
</flux>
</bastion>"""


def test_extremite():
    """
    Les extrémités correspondant à la source et à la destination sont présentes
    la source
    >>> result = transform(XML_EXTREMITE)
    >>> document = microdom.parseString(result)
    >>> extremite = domhelpers.findNodesNamed(document.documentElement, 'extremite')[0]
    >>> extremite.hasAttribute("name")
    True
    >>> extremite.getAttribute("name")
    '%%nom_machine_eth1%%'

    la destination
    >>> extremite = domhelpers.findNodesNamed(document.documentElement, 'extremite')[1]
    >>> extremite.hasAttribute("name")
    True
    >>> extremite.getAttribute("name")
    'bastion'
    
    """

def test_extremite_redirige():
    """fixme L'extremité correspondant au redirige est présente
    """

XML_SUBNET = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<pont id = "connexion_dns" ordre = "130" libelle = "connexions au dns" optionnel="non">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<port>53</port>
	</service> 
</pont>    
</montant>
</flux>
</bastion>"""


def test_subnet():
    """test de attribut subnet  (machine ou sous-réseau)

    Les extrémités correspondant à la source et à la destination sont présentes
    la source
    >>> result = transform(XML_SUBNET)
    >>> document = microdom.parseString(result)
    >>> extremite = domhelpers.findNodesNamed(document.documentElement, 'extremite')[0]
    >>> extremite.hasAttribute("subnet")
    True
    >>> extremite.getAttribute("subnet")
    '1'
    """

XML_ACTION1 = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<pont id = "connexion_dns" ordre = "130" libelle = "connexions au dns" optionnel="non">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<port>53</port>
	</service> 
</pont>    
</montant>
</flux>
</bastion>"""

XML_ACTION2 = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<descendant>
<barrage id = "connexion_dns" ordre = "130" libelle = "connexions au dns" optionnel="non">    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
        <!-- ici la directive doit être inversée -->
		<ip>! gwen_0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>        
	<service id = "dns">    
		<protocole>tcp</protocole>    
		<port>53</port>
	</service> 
</barrage>    
</descendant>
</flux>
</bastion>"""

def test_action():
    """Test de l'action pour une directive montante
    # ACTION_DENY = 1 : barrage
    # ACTION_ALLOW = 2 : pont

    >>> result = transform(XML_ACTION1)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.hasAttribute("action")
    True
    >>> directive.getAttribute("action")
    '2'


    directive descendante

    >>> result = transform(XML_ACTION2)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.hasAttribute("action")
    True
    >>> directive.getAttribute("action")
    '1'

    """


XML_REDIRECT = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<redirect>    
	<source id = "bastion" extremite = "%%nom_machine_eth2%%">    
		<ip>! %%adresse_ip_eth2%%</ip>
		<netmask>255.255.255.255</netmask>
		<interface>eth2</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    	    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>    
	<translation id = "proxy">    
		<protocole>tcp</protocole>    
		<de>3128</de>
		<vers>3128</vers>    
	</translation>
</redirect>    
</montant>
</flux>
</bastion>"""


def test_redirect():
    """
    >>> result = transform(XML_REDIRECT)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.hasAttribute("nat_port")
    True
    >>> directive.getAttribute("nat_port")
    '3128'
    >>> directive.getAttribute("service")
    'proxy'

    """

XML_DNAT = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<dnat>
	<source id = "tout" extremite = "exterieur">
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>eth0</interface>
	</source>
	<destination id = "mailweb" extremite = "bastion">
		<ip>217.167.50.3</ip>
		<netmask>255.255.255.255</netmask>
		<interface>eth0</interface>
	</destination>
	<translation id = "https">
		<protocole>tcp</protocole>
		<de>443</de>
	</translation>
	<redirige id = "mailweb" extremite = "%%nom_machine_eth3%%">
		<ip>172.18.0.10</ip>
		<netmask>255.255.255.255</netmask>
		<interface>eth3</interface>
	</redirige>
</dnat>
</montant>
</flux>
</bastion>"""

def test_dnat():
    """
    >>> result = transform(XML_DNAT)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.hasAttribute("nat_port")
    True
    >>> directive.getAttribute("nat_port")
    '443'
    >>> directive.getAttribute("service")
    'https'
    >>> directive.getAttribute("nat_extr")
    'mailweb'
    """

XML_MASQUE = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<descendant>
<masque id = "" libelle = "">
	<source id = "webpub" extremite = "%%nom_machine_eth3%%">
		<ip>172.18.0.10</ip>
		<netmask>255.255.255.255</netmask>
		<interface>eth3</interface>
	</source>
	<destination id = "tout" extremite = "exterieur">
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>eth0</interface>
	</destination>
	<translation id = "ftp">
		<protocole>tcp</protocole>
		<de>21</de>
	</translation>
	<redirige id = "bastion" extremite = "exterieur">
		<ip>%%adresse_ip_eth0%%</ip>
		<netmask>255.255.255.255</netmask>
		<interface>eth0</interface>
	</redirige>
</masque>

</descendant>
</flux>
</bastion>"""

def test_masque():
    """
    >>> result = transform(XML_MASQUE)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.hasAttribute("nat_port")
    True
    >>> directive.getAttribute("nat_port")
    '21'
    >>> directive.getAttribute("service")
    'ftp'
    >>> directive.getAttribute("nat_extr")
    'bastion'
    """


XML_ICMP = """<?xml version = "1.0" encoding = "ISO-8859-1"?>
<bastion nom = "pare-feu-3cartes">
<flux nom = "bastion-%%nom_machine_eth1%%">
<montant>
<icmp>    
	<source id = "%%nom_machine_eth1%%" extremite = "%%nom_machine_eth1%%">    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</source>    
	<destination id = "bastion" extremite = "bastion">    	    
		<ip>0</ip>
		<netmask>0</netmask>
		<interface>0</interface>    
	</destination>    
	<ping>echo-request</ping>
</icmp>
</montant>
</flux>
</bastion>"""

def test_icmp():
    """
    >>> result = transform(XML_ICMP)
    >>> document = microdom.parseString(result)
    >>> directive = domhelpers.findNodesNamed(document.documentElement, 'directive')[0]
    >>> directive.getAttribute("service")
    'echo-request'
    >>> directive.getAttribute("action")
    '2'
    """



def test_suite():
    import sys
    import doctest
    return doctest.DocTestSuite(sys.modules[__name__])

if __name__ == "__main__":
    import unittest
    unittest.main(defaultTest='test_suite')
