from starqos.qosconsumer import load_from_file, InboundQos, OutboundQos

def getDirectiveZoneSrc(directive):
    return directive.src_list[0].zone

def getDirectiveZoneDst(directive):
    return directive.dest_list[0].zone

class QoS:
    def __init__(self, filename):
        self.starqos = load_from_file(filename)

    def getRule(self, qos_type, directive):
        """
        Get the QoS rule relative to a directive (qos_in or qos_out).
        qos_type is "in" to use qos_in or "out" for qos_out.
        """
        if qos_type == "in":
            qos_id = directive.qos_in
        else:
            qos_id = directive.qos_out
        qos_rule = self.starqos.get_rule(qos_id)

        # Check way (input or ouput)
        bound = qos_rule.get_containing_qos()
        if qos_type == "in":
            assert isinstance(bound, InboundQos)
        else:
            assert isinstance(bound, OutboundQos)

        # Check interface
        qos_interface = bound.get_containing_interface().get_name()
        if directive.is_montante():
            assert qos_interface == getDirectiveZoneSrc(directive).interface, "Interface: %s != %s" % (qos_interface, getDirectiveZoneSrc(directive).interface)
        else:
            assert qos_interface == getDirectiveZoneDst(directive).interface, "Interface: %s != %s" % (qos_interface, getDirectiveZoneDst(directive).interface)

        return qos_rule


