# -*- coding: UTF-8 -*-
"""
Feuille de style de traduction au format RML
(http://openreport.org/index.py/trml2pdf)
"""

from trml2pdf.trml2pdf import parseString

from os.path import splitext

def dump(filename, rml_string):
    RML(filename).dump_pdf(rml_string)

PAGE = """<?xml version="1.0" encoding="iso-8859-1" ?>                                    
<!DOCTYPE document SYSTEM "rml_1_0.dtd">
<document filename="">
<!-- template pageSize="(21cm, 29.7cm)" -->
<template pageSize="(29.7cm, 21cm)" 
          leftMargin="1.5cm" 
          rightMargin="1.5cm" 
          topMargin="1.5cm" 
          bottomMargin="1.5cm" 
          showBoundary="1" 
          allowSplitting="20" 
          >  
<pageTemplate id="main">
  <frame id="first" x1="1.5cm" y1="1.5cm" width="19cm" height="25.5cm"/>
</pageTemplate>
</template>

<stylesheet>

<blockTableStyle id="myBlockTableStyle" align="left">
  <blockAlignment value="RIGHT" start="-1,0" stop="-1,-1"/>
  <blockFont name="Helvetica-BoldOblique" size="10" start="0,0" stop="1,0"/>
  <blockFont name="Courier-Bold" size="10" start="0,1" stop="-1,-1"/>
  <lineStyle kind="GRID" colorName="silver"/> 
</blockTableStyle>              

<paraStyle name="bullet" fontName="Helvetica" fontSize="10" leftIndent="10mm" firstLineIndent="-10mm"/>

</stylesheet>
<story>
<image file="images/titre.png" x="20" y="20" width="466" height="66"/>
<!-- width="59" height="71"/ -->
<title>%(title)s</title>
<spacer length="1cm"/>
%(story)s
</story>
</document>
"""

def para(content):
    """Formatage d'un paragraphe
    """
    PARA = """<para>%s</para>"""
    return PARA % content

def title1(content):
    """Formatage d'un titre
    """
    return _title(1) % content


def title2(content):
    """Formatage d'un titre
    """
    return _title(2) % content

def title3(content):
    """Formatage d'un titre
    """
    return _title(3) % content

def _title(type):
    type = str(type)
    TITLE = """<spacer length="15"/><h%s>%%s</h%s><spacer length="10"/>"""
    return TITLE % (type, type)

def page_break():
    return """<condPageBreak height="200cm"/>"""
    
class RML:
    """Conversion et sérialisation en RML
    """
    def __init__(self, filename):
        self.filename = filename
        self.pdf_filename = splitext(filename)[0] + ".pdf"
        
    def dump_pdf(self, rml_string):
        """Conversion du rml en pdf
        """
        fh = file(self.pdf_filename, "w")

        # modele de page
        TITLE = "Rapport du parefeu sur le fichier : %s"
        rml_page = self.page(TITLE % (self.filename) , rml_string) 
        # ecriture du pdf
        # FIXME -- ecriture du rml pour l'instant 
        # fh.write( rml_page )
        fh.write( parseString(rml_page) )
                
        fh.close()

    def page(self, title, story):
        """Modèle de génération d'une page
        """
        return PAGE % {"title":title, "story":story}


if __name__ == "__main__":
    pass
    
    
